/*
 * My RSVP reader.
 * http://www.mscottreynolds.com
 *
Copyright (c) 2003, 2004, 2005, M. Scott Reynolds
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, 
      this list of conditions and the following disclaimer in the documentation 
      and/or other materials provided with the distribution.
    * Neither the name of M. Scott Reynolds nor the names of its contributors 
      may be used to endorse or promote products derived from this software
      without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class MyRSVP extends Applet
{
  public static final int WORDS_PER_MINUTE = 300;
  public static final int MIN_WORD_LENGTH = 1;
  public static final int MAX_WORD_LENGTH = 10;

  private Button startStop;
  private TextField wordsPerMinute;
  private TextField minWordLength;
  private TextField maxWordLength;
  private Label wordDisplay;
  private TextArea textInput;
  private Button clearButton;
  private Label statusLabel;
  private MyRSVPThread thread;

  public Label getWordDisplay()
  {
    return wordDisplay;
  }

  public TextArea getTextInput()
  {
    return textInput;
  }

  // Do applet initialization
  public void init()
  {
    setLayout(new BorderLayout());

    // Setup the toolbar
    Panel toolBar = new Panel();
    toolBar.setLayout(new FlowLayout(FlowLayout.LEFT));
    startStop = new Button("Start");
    startStop.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent evt)
      {
	doStartStop();
      }
    });

    toolBar.add(startStop);
    toolBar.add(new Label("Words Per Minute:"));
    wordsPerMinute = new TextField(5);
    wordsPerMinute.setText(String.valueOf(WORDS_PER_MINUTE));
    wordsPerMinute.setBackground(Color.white);
    toolBar.add(wordsPerMinute);

    toolBar.add(new Label("Min Word Length:"));
    minWordLength = new TextField(2);
    minWordLength.setText(String.valueOf(MIN_WORD_LENGTH));
    minWordLength.setBackground(Color.white);
    toolBar.add(minWordLength);

    toolBar.add(new Label("Max Word Length:"));
    maxWordLength = new TextField(2);
    maxWordLength.setText(String.valueOf(MAX_WORD_LENGTH));
    maxWordLength.setBackground(Color.white);
    toolBar.add(maxWordLength);

    clearButton = new Button("Clear");
    clearButton.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent evt)
      {
	wordDisplay.setText("");
	textInput.setText("");
      }
    });
    toolBar.add(clearButton);
    add(toolBar, BorderLayout.NORTH);

    // Setup the bottom panel
    Panel bottomPanel = new Panel();
    bottomPanel.setLayout(new GridLayout(2,1));
    wordDisplay = new Label("");
    wordDisplay.setFont(new Font("Default", Font.BOLD, 24));
    wordDisplay.setBackground(Color.black);
    wordDisplay.setForeground(Color.yellow);
    //wordDisplay.setBackground(Color.white);
    //wordDisplay.setForeground(Color.black);
    wordDisplay.setAlignment(Label.CENTER);
    wordDisplay.addMouseListener(new MouseAdapter()
    {
      public void mousePressed(MouseEvent evt)
      {
	doStartStop();
      }
    });

    bottomPanel.add(wordDisplay);
    textInput = new TextArea("", 12, 48, TextArea.SCROLLBARS_VERTICAL_ONLY);
    textInput.setBackground(Color.white);
    bottomPanel.add(textInput);
    add(bottomPanel, BorderLayout.CENTER);

    // Create the status label
    statusLabel = new Label("Ready");
    add(statusLabel, BorderLayout.SOUTH);
  }

  // When this is run as an applet and is initialized, this will be called.
  public void start()
  {
    textInput.setText("The quick brown fox jumped over the lazy dogs.");
    doStartStop();
  }

  // Start/Stop the display of text.
  public synchronized void doStartStop()
  {
    if (thread == null || !thread.isAlive())
    {
      int wpm = parseIntDef(wordsPerMinute.getText(), WORDS_PER_MINUTE);
      wordsPerMinute.setText(String.valueOf(wpm));
      int min = parseIntDef(minWordLength.getText(), MIN_WORD_LENGTH);
      minWordLength.setText(String.valueOf(min));
      int max = parseIntDef(maxWordLength.getText(), MAX_WORD_LENGTH);
      maxWordLength.setText(String.valueOf(max));
      thread = new MyRSVPThread(this, wpm, min, max);
      thread.start();
      updateControls(false);
    }
    else
    {
      thread.doStop();
      updateControls(true);
      thread = null;
    }
  }

  public int parseIntDef(String value, int defaultValue)
  {
    try
    {
      return Integer.parseInt(value);
    }
    catch (Exception e)
    {
      return defaultValue;
    }
  }

  // Update the controls.
  public synchronized void updateControls(boolean stopped)
  {
    startStop.setLabel(stopped ? "Start" : "Stop");
    wordsPerMinute.setEnabled(stopped);
    minWordLength.setEnabled(stopped);
    maxWordLength.setEnabled(stopped);
    textInput.setEditable(stopped);
    clearButton.setEnabled(stopped);
    statusLabel.setText(stopped ? "Ready" : "Displaying at " + wordsPerMinute.getText() + " words per minute...");
//    if (stopped)
//      thread = null;
  }

  // Main
  public static void main(String argv[])
  {
    final Frame win = new Frame("My RSVP");
    MyRSVP applet = new MyRSVP();
    applet.init();
    win.add(applet);
    win.addWindowListener(new WindowAdapter()
      {
	public void windowClosing(WindowEvent evt)
	{
	  win.dispose();
	  System.exit(0);
	}
      });
    win.pack();
    win.setVisible(true);
  }
}

class MyRSVPThread extends Thread
{
  private MyRSVP app;
  private boolean stopped;
  private int wpm;
  private Label wordDisplay;
  private TextArea textInput;
  private int minWordLength;
  private int maxWordLength;

  public MyRSVPThread(MyRSVP app, int wpm, int min, int max)
  {
    this.app = app;
    this.wpm = wpm;
    wordDisplay = app.getWordDisplay();
    textInput = app.getTextInput();
    minWordLength = min;
    maxWordLength = max;
  }

  public synchronized void doStop()
  {
    stopped = true;
  }

  // Display the text.
  public void run()
  {
    String text = textInput.getText();
    int textLength = text.length();
    int i = textInput.getCaretPosition();
    int delay = 60000 / wpm;

    while (i < textLength && !stopped)
    {

      // Skip leading whitespace
      while (i < textLength && text.charAt(i) <= ' ')
	i++;

      if (i < textLength)
      {
	// Find end of current word
	int j = i;
	while (j < textLength && (text.charAt(j) > ' ' || j-i < minWordLength))
	  j++;

	// Maxed out on the word length?
	if (j-i > maxWordLength)
	{
	  int n = j-1;
	  while (n > i && text.charAt(n) > ' ' && text.charAt(n) != '-')
	    n--;

	  // Was there a word break found?
	  if (n > i)
	    j = n;
	}

	// Get the word(s) and convert any non printable chars to spaces.
	StringBuffer words = new StringBuffer(text.substring(i, j));
	for (int n = 0; n < words.length(); n++)
	  if (words.charAt(n) < ' ')
	    words.setCharAt(n, ' ');

	// Display the word(s).
	wordDisplay.setText(words.toString());
	textInput.setSelectionStart(i);
	textInput.setSelectionEnd(j);
	app.repaint();

	try
	{
	  sleep(delay);
	}
	catch (InterruptedException ie)
	{
	}

	i = j+1;
      }
    }
    app.updateControls(true);
  }
}
